# coding=utf-8
"""
Convenience functions to manage logging
"""
from elib.custom_logging._custom_logging_handler import CustomLoggingHandler
from ._custom_logging import activate_elib_logging, get_logger, set_handler_level, get_elib_logger
